package com.vandyo.sentry.core.tools;

import cn.hutool.crypto.SecureUtil;
import com.vandyo.sentry.core.Tmp;
import com.vandyo.sentry.core.dto.ErrStatus;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class Signature {
    private static final Logger log = LoggerFactory.getLogger(Tmp.class);
    // need be change to product env
//    public static final String host = "http://10.100.11.41:8080";
    public static final String host = "http://10.100.12.42:8080";

    /**
     * 生成签名
     *
     * @param map
     * @return
     */
    public static String calcChecksum(Map<String, String> map) {
        String s = new String();
        Set<String> set = map.keySet();
        Iterator<String> iter = set.iterator();
        while (iter.hasNext()) {
            String key = iter.next();
            String value = map.get(key);
            s += key;
            s += "[";
            s += value;
            s += "];";
        }
        try {
            return SecureUtil.md5(s);
        } catch (Exception e) {
            log.error("计算校验码发生异常", e);
            return null;
        }
    }


    /**
     * 对数据进行签名
     * @param data
     * @return
     */
    public static Map<String,String> sign(Map<String,String> data,String sid){
        if (!data.containsKey("ts")){
            data.put("ts",String.valueOf(System.currentTimeMillis()/1000));
        }

        Map<String,String> order = new TreeMap<>(data);
        if(Objects.nonNull(sid)){
            order.put("sid",sid);
        }
        String cs = Signature.calcChecksum(order);
        order.put("cs",cs);
        order.remove("sid");
        return order;
    }

    public static ErrStatus matchStatus(int status) {
        if (status == 408){
            return ErrStatus.ErrTimeout;
        }
        if (status >= 400 && status < 500) {
            return ErrStatus.Err4xx;
        } else if (status >= 500 && status < 600) {
            return ErrStatus.Err5xx;
        } else {
            return ErrStatus.ErrUnknown;
        }
    }

    /**
     * 将map转换成url
     *
     * @param map
     * @return
     */
    public static String getUrlParamsByMap(Map<String, String> map) {
        if (map == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        for (Map.Entry<String, String> entry : map.entrySet()) {
            sb.append(entry.getKey() + "=" + entry.getValue());
            sb.append("&");
        }
        String s = sb.toString();
        if (s.endsWith("&")) {
            s = StringUtils.substringBeforeLast(s, "&");
        }
        return s;
    }

    public static void main(String[] args) {
        Map<String,String> map = new HashMap<>();
        map.put("ts","1584066174");
        map.put("uid","860a776c832749ce86754d59c7ca9fca");

//        cs d429d092bd3d30e9151c6f48cfbaaa2b
//         ts ts -> 1584066174
//          uid -> 860a776c832749ce86754d59c7ca9fca
//        map.put("sid","dcfab1c8d7fe4797854b845d90da1054");
//        map.put("ts","1576209501");
//        map.put("uid","417126c931e14964bade3095b729a5fd");
//        System.out.println("393ddcc7b76ab7ef5bc79a9f0d2f04f8");
        System.out.println(sign(map,null).get("cs"));
    }

}
